""" Main application entry point.

    python -m guanoctl  ...

"""
def main():
    """ Execute the application.

    """
    raise NotImplementedError


# Make the script executable.

if __name__ == "__main__":
    raise SystemExit(main())
